# Generated by Django 4.2.11 on 2024-03-27 09:44

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("wbfdm", "0013_instrument_is_investable_universe_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="controversy",
            name="instrument",
            field=models.ForeignKey(
                limit_choices_to=models.Q(("level", 0)),
                on_delete=django.db.models.deletion.CASCADE,
                to="wbfdm.instrument",
            ),
        ),
    ]
