# Generated by Django 4.2.6 on 2023-10-30 15:30

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("wbfdm", "0008_controversy"),
    ]

    operations = [
        migrations.AlterField(
            model_name="controversy",
            name="flag",
            field=models.CharField(
                blank=True,
                choices=[
                    ("GREEN", "Green"),
                    ("YELLOW", "Yellow"),
                    ("ORANGE", "Orange"),
                    ("RED", "Red"),
                    ("UNKNOWN", "Unknown"),
                ],
                default="GREEN",
                max_length=64,
                null=True,
                verbose_name="Flag",
            ),
        ),
        migrations.AlterField(
            model_name="controversy",
            name="initiated",
            field=models.DateField(blank=True, null=True, verbose_name="initiated"),
        ),
        migrations.AlterField(
            model_name="controversy",
            name="severity",
            field=models.CharField(
                blank=True,
                choices=[
                    ("MINOR", "Minor"),
                    ("MODERATE", "Moderate"),
                    ("SEVERE", "Severe"),
                    ("VERY_SEVERE", "Very Severe"),
                ],
                default="MINOR",
                max_length=64,
                null=True,
                verbose_name="Severity",
            ),
        ),
        migrations.AlterField(
            model_name="controversy",
            name="source",
            field=models.CharField(blank=True, max_length=512, null=True, verbose_name="Source"),
        ),
        migrations.AlterField(
            model_name="controversy",
            name="status",
            field=models.CharField(
                blank=True,
                choices=[
                    ("ONGOING", "Ongoing"),
                    ("PARTIALLY_CONCLUDED", "Partially Concluded"),
                    ("CONCLUDED", "Concluded"),
                ],
                default="ONGOING",
                max_length=64,
                null=True,
                verbose_name="Status",
            ),
        ),
        migrations.AlterField(
            model_name="controversy",
            name="type",
            field=models.CharField(
                blank=True,
                choices=[("STRUCTURAL", "Structural"), ("NON_STRUCTURAL", "Non Structural")],
                default="STRUCTURAL",
                max_length=64,
                null=True,
                verbose_name="Type",
            ),
        ),
    ]
