# Generated by Django 5.0.10 on 2025-02-03 10:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wbfdm', '0028_instrumentprice_annualized_daily_volatility'),
    ]

    operations = [
        migrations.AlterField(
            model_name='instrumentprice',
            name='volume',
            field=models.FloatField(default=0.0, help_text='The Volume of the Asset on the price date of the Asset.', verbose_name='Volume'),
        ),
    ]
