# Generated by Django 5.0.14 on 2025-07-02 07:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wbfdm', '0030_alter_relatedinstrumentthroughmodel_related_type'),
    ]

    operations = [
        migrations.AddField(
            model_name='exchange',
            name='apply_round_lot_size',
            field=models.BooleanField(default=True, help_text='If False, the quotes linked to this instrument will ignore their round lot size (will be considered 1).', verbose_name='Apply Round Lot Size'),
        ),
        migrations.AddField(
            model_name='instrument',
            name='round_lot_size',
            field=models.IntegerField(default=1, help_text='A round lot (or board lot) is the normal unit of trading of a security.', verbose_name='Round Lot Size'),
        ),
    ]
